@@ -1,15 +1,25 @@
+##############################################################################
+# Imports
+##############################################################################
 
-
-import os
 import os.path
 
 import sys
 import stat  # file permissions
 import argparse
 from argparse import RawTextHelpFormatter
-import tempfile
 import shutil
+
+##############################################################################
+# Local imports
+##############################################################################
+
 import console
+import common
+
+##############################################################################
+# Methods
+##############################################################################
 
 
 def help_string():
@@ -23,16 +33,6 @@ def help_string():
     return overview + instructions
 
 
-def create_groovy_script(script_name):
-    f = tempfile.NamedTemporaryFile(delete=False)
-    f.write("#!/bin/bash\n")
-    f.write("source /opt/ros/groovy/setup.bash\n")
-    f.write(script_name + "\n")
-    os.chmod(f.name, stat.S_IRWXU)
-    f.close()
-    return f
-
-
 def parse_arguments():
     parser = argparse.ArgumentParser(description=help_string(), formatter_class=RawTextHelpFormatter)
     parser.add_argument('dir', nargs='?', default="./", help='directory to use for the parallel development space [./]')
@@ -44,6 +44,9 @@ def parse_arguments():
     parser.add_argument('-p', '--platform', action='store', default='', help='platform cmake cache module to load []')
     parser.add_argument('--list-toolchains', action='store_true', help='list all currently available toolchain modules [false]')
     parser.add_argument('--list-platforms', action='store_true', help='list all currently available platform modules [false]')
+    parser.add_argument('--track', action='store', default=None, help='retrieve rosinstalls relevant to this track [groovy|hydro][groovy]')
+    parser.add_argument('--get-default-track', action='store_true', help='print the default track that is being followed to screen')
+    parser.add_argument('--set-default-track', action='store', default=None, help='set a new default track to work from %s' % common.VALID_TRACKS)
     args = parser.parse_args()
     return args
 
@@ -90,16 +93,35 @@ def instantiate_template(filename, name, cwd):
         f.close()
 
 
-def fill_in_makefile(template, cwd, source_dir, underlays, install_prefix, build_type, toolchain, platform):
+def fill_in_makefile(template, rel_source_dir, catkin_make, catkin_python_path, toolchain, override):
     return template % locals()
 
 
-def instantiate_makefile(filename, cwd, source_dir, underlays, install_prefix, build_type, toolchain, platform):
+def instantiate_makefile(filename, build_dir, rel_source_dir, catkin_make, catkin_python_path, toolchain):
     template_dir = os.path.join(os.path.dirname(__file__), 'templates', 'init_build')
     tmpl = read_template(os.path.join(template_dir, filename))
-    contents = fill_in_makefile(tmpl, cwd, source_dir, underlays, install_prefix, build_type, toolchain, platform)
+    contents = fill_in_makefile(tmpl, rel_source_dir, catkin_make, catkin_python_path, toolchain, common.override_filename())
     try:
-        f = open(os.path.join(cwd, filename), 'w')
+        f = open(os.path.join(build_dir, filename), 'w')
+        f.write(contents.encode('utf-8'))
+    finally:
+        f.close()
+
+
+def fill_in_config_cmake(template, config_install_prefix, config_underlays):
+    return template % locals()
+
+
+def instantiate_config_cmake(build_path, config_install_prefix, config_underlays):
+    '''
+      Copy the cache configuration template to the build path.
+    '''
+    template_dir = os.path.join(os.path.dirname(__file__), 'cmake')
+    template = read_template(os.path.join(template_dir, "config.cmake"))
+    contents = fill_in_config_cmake(template, config_install_prefix, config_underlays)
+    config_cmake_file = os.path.join(build_path, "config.cmake")
+    try:
+        f = open(config_cmake_file, 'w')
         f.write(contents.encode('utf-8'))
     finally:
         f.close()
@@ -152,6 +174,7 @@ def init_configured_build(build_dir_="./", source_dir_="./src", underlays_="/opt
     '''
     # Help us build the name for the eclipse workspace...usually we call in the workspace itself.
     workspace_dir = os.getcwd()
+
     ##########################
     # Build directory
     ##########################
@@ -168,29 +191,58 @@ def init_configured_build(build_dir_="./", source_dir_="./src", underlays_="/opt
     else:
         console.logerror("This build directory is already initialised")
         sys.exit(1)
+
     ##########################
     # Source directory
     ##########################
-    if os.path.isabs(source_dir_):
-        source_dir = source_dir_
-    else:
-        source_dir = os.path.join(os.getcwd(), source_dir_)
+    # Absolute path - should probably use os.path.abspath here
+    #if os.path.isabs(source_dir_):
+    #    source_dir = source_dir_
+    #else:
+    #    source_dir = os.path.join(os.getcwd(), source_dir_)
+    source_dir = os.path.abspath(source_dir_)
+    build_source_dir = os.path.join(build_dir, 'src')
     if not os.path.isdir(source_dir):
-        console.logerror("Specified source folder does not exist [" + source_dir + "]")
+        console.logerror("Specified source space does not exist [" + source_dir + "]")
         sys.exit(1)
     if not os.path.isfile(os.path.join(source_dir, ".rosinstall")):
         console.logerror("Could not find a valid source folder (must contain a .rosinstall file therein)'")
         sys.exit(1)
+    if os.path.exists(build_source_dir):
+        if not source_dir == build_source_dir:
+            console.error("The build directory already has a ./src directory which doesn't match the desired source directory [%s]" % source_dir)
+            sys.exit(1)
+    else:  # create a symlink to the sources
+        common.symlink_dir(source_dir, build_source_dir)
+
     ##########################
     # Underlays
     ##########################
-    underlays = underlays_
-    if underlays_.find("/opt/ros/groovy") == -1:
-        if underlays_ == "":
-            underlays = "/opt/ros/groovy"
+    env_underlays = os.environ['CMAKE_PREFIX_PATH']
+    underlays_list = underlays_.split(';')
+    env_underlays_list = env_underlays.split(':')
+    for underlay in env_underlays_list:
+        if underlay not in underlays_list:
+            underlays_list.append(underlay)
+    catkin_found = False
+    for underlay in underlays_list:
+        if os.path.isfile(os.path.join(underlay, 'bin', 'catkin_make')):
+            catkin_make = os.path.join(underlay, 'bin', 'catkin_make')
+            catkin_python_path = os.path.join(underlay, 'lib', 'python2.7', 'dist-packages')
+            catkin_found = True
+            break
+    if not catkin_found:
+        # Add the default track underlay
+        default_track = common.get_default_track()
+        if os.path.isfile(os.path.join("/opt/ros/%s" % default_track, 'bin', 'catkin_make')):
+            console.pretty_println("No catkin found, adding the default track underlay [/opt/ros/%s]" % default_track)
+            underlays_list.append("/opt/ros/%s" % default_track)
+            catkin_make = os.path.join("/opt/ros/%s" % default_track, 'bin', 'catkin_make')
+            catkin_python_path = os.path.join("/opt/ros/%s" % default_track, 'lib', 'python2.7', 'dist-packages')
         else:
-            #underlays = "/opt/ros/groovy;" + underlays_
-            underlays = underlays_ + ";/opt/ros/groovy"
+            console.logerror("Could not find an underlying catkin installation.")
+    underlays = ';'.join(underlays_list)
+
     ##########################
     # Toolchain
     ##########################
@@ -198,17 +250,16 @@ def init_configured_build(build_dir_="./", source_dir_="./src", underlays_="/opt
     if not toolchain_ == "":
         toolchains_dir = os.path.join(os.path.dirname(__file__), 'toolchains')
         if os.path.isfile(os.path.join(toolchains_dir, toolchain_ + ".cmake")):
-            shutil.copy(os.path.join(toolchains_dir, toolchain_ + ".cmake"), build_dir)
-            toolchain = '-DCMAKE_TOOLCHAIN_FILE=' + toolchain_ + ".cmake"
+            shutil.copy(os.path.join(toolchains_dir, toolchain_ + ".cmake"), os.path.join(build_dir, "toolchain.cmake"))
+            toolchain = toolchain = "-DCMAKE_TOOLCHAIN_FILE=toolchain.cmake"
+
     ##########################
     # Platform
     ##########################
-    platform = platform_
     if not platform_ == "":
         platforms_dir = os.path.join(os.path.dirname(__file__), 'platforms')
         if os.path.isfile(os.path.join(platforms_dir, platform_ + ".cmake")):
-            shutil.copy(os.path.join(platforms_dir, platform_ + ".cmake"), build_dir)
-            platform = '-C ' + platform_ + ".cmake"
+            shutil.copy(os.path.join(platforms_dir, platform_ + ".cmake"), os.path.join(build_dir, "platform.cmake"))
 
     ##########################
     # Other Args
@@ -226,6 +277,11 @@ def init_configured_build(build_dir_="./", source_dir_="./src", underlays_="/opt
     print_build_details(build_dir, source_dir, install_prefix, build_type, underlays, name, toolchain_, platform_)
     os.chdir(build_dir)
 
+    ##########################
+    # Cache
+    ##########################
+    instantiate_config_cmake(build_dir, install_prefix, underlays)
+
     ##########################
     # Templates
     ##########################
@@ -233,11 +289,27 @@ def init_configured_build(build_dir_="./", source_dir_="./src", underlays_="/opt
     instantiate_template('konsole', name, build_dir)
     instantiate_template('gnome-terminal', name, build_dir)
     instantiate_template('eclipse', name, build_dir)
-    instantiate_makefile('Makefile', os.path.relpath(build_dir, os.getcwd()), os.path.relpath(source_dir, build_dir), underlays, install_prefix, build_type, toolchain, platform)
+    instantiate_makefile('Makefile', build_dir, os.path.relpath(source_dir, build_dir), catkin_make, catkin_python_path, toolchain)
 
 
 def init_build():
     args = parse_arguments()
+    ##########################
+    # Tracks
+    ##########################
+    if args.get_default_track:
+        console.pretty_print("\nDefault Track: ", console.cyan)
+        console.pretty_println("%s\n" % common.get_default_track(), console.yellow)
+        sys.exit(0)
+    if args.set_default_track:
+        console.pretty_print("\nNew Default Track: ", console.cyan)
+        console.pretty_println("%s\n" % common.set_default_track(args.set_default_track), console.yellow)
+        sys.exit(0)
+    if not args.track:
+        args.track = common.get_default_track()
+    ##########################
+    # Toolchains and Platform
+    ##########################
     if args.list_toolchains:
         list_toolchains()
         sys.exit(0)
